unit dmserver;

{$mode ObjFPC}

interface

uses
  Classes, SysUtils, fprpcclient, services, sqldbrestdataset;

Type
  TUserInfo = record
    Login : String;
    ID : NativeInt;
  end;

  { TServerModule }

  TServerModule = Class(TComponent)
  private
    FOnInvalidLogin: TNotifyEvent;
    FOnLogin: TNotifyEvent;
    FRestConnection: TSQLDBRestConnection;
    FUserInfo: TUserInfo;
    FClient:TRPCClient;
    FUserService: TUserService;
  Public
    Constructor Create(aOwner : TComponent); override;
    destructor Destroy; override;
    Procedure DoLogin(const aUserName,aPassword : String);
    Property UserService : TUserService Read FUserService;
    Property UserInfo : TUserInfo Read FUserInfo;
    Property OnInvalidLogin : TNotifyEvent Read FOnInvalidLogin Write  FOnInvalidLogin;
    Property OnLogin : TNotifyEvent Read FOnLogin Write FOnLogin;
    Property RestConnection : TSQLDBRestConnection Read FRestConnection;
  end;

Var
  Server : TServerModule;

implementation

{ TServerModule }


constructor TServerModule.Create(aOwner: TComponent);

Const
  ServerURL = 'http://localhost:8080/';

begin
  inherited Create(aOwner);
  FUserService:=TUserService.Create(Self);
  FClient:=TRPCClient.Create(Self);
  FClient.URL:=ServerURL+'RPC/';
  FUserService.RPCClient:=Fclient;
  FRestConnection:=TSQLDBRestConnection.create(Self);
  FRestConnection.BaseURL:=ServerURL+'REST/';
end;

destructor TServerModule.Destroy;
begin
  FreeAndNil(FUserService);
  FreeAndNil(FRestConnection);
  inherited Destroy;
end;

procedure TServerModule.DoLogin(const aUserName, aPassword: String);

  procedure DoOK(aResult: NativeInt);
  begin
    FUserInfo.Login:=aUserName;
    FUserInfo.ID:=aResult;
    if Assigned(FOnLogin) then
      FonLogin(Self);
  end;

  procedure DoFail(Sender: TObject; const aError: TRPCError);
  begin
    FUserInfo.Login:='';
    FUserInfo.ID:=-1;
    If Assigned(FOnInvalidLogin) then
      FOnInvalidLogin(Self);
  end;

begin
  UserService.Login(aUserName,aPassword,@DoOK,@DoFail);
end;

end.

