unit frmarticle;
{$MODE ObjFPC}
{$H+}

interface

uses js, sysutils, web, Classes, types, frmbase, db, webrouter, sqldbrestdataset, dbwebwidget;

type
  
  { --------------------------------------------------------------------
    TArticleForm
    --------------------------------------------------------------------}
  
  { TArticleForm }

  TArticleForm = class(TBaseForm)
  Published
    divNoArticles : TJSHTMLElement;
    divTitle : TJSHTMLElement;
    lblTitle : TJSHTMLElement;
    lblAuthor : TJSHTMLElement;
    lblDate : TJSHTMLElement;
    divEditTitle : TJSHTMLElement;
    edtTitle : TJSHTMLInputElement;
    divArticleBlock : TJSHTMLElement;
    divArticle : TJSHTMLElement;
    btnSave : TJSHTMLButtonElement;
    divComments : TJSHTMLElement;
    divExistingComments : TJSHTMLElement;
    editcomment : TJSHTMLElement;
    btnAddComment : TJSHTMLButtonElement;
    procedure HandleSavePost(Event : TJSEvent); virtual;
    procedure HandleSaveComment(Event : TJSEvent); virtual;
  Private
    FArticleID : NativeInt;
    rdsArticle: TSQLDBRestDataset;
    rdsComments: TSQLDBRestDataset;
    dsComments : TDatasource;
    ltwComments : TDBLoopTemplateWidget;
    procedure DoAfterArticleOpen(DataSet: TDataSet);
    class procedure HideEl(aEl: TJSHTMLElement);
    procedure SaveData(Dataset: TDataset);
    Procedure SetupDatasets;
    procedure SetupForEdit(IsEditing: Boolean);
    Procedure SetupTemplates;
    procedure ShowData(Dataset: TDataset);
    class procedure ShowEl(aEl: TJSHTMLElement);
  Public
    constructor create(aOwner : TComponent); override;
    procedure BindElements; virtual;
    procedure BindElementEvents; virtual;
    Procedure ShowRoute(Const aURL : String; aRoute : TRoute; aParams : TStrings); override;
    class function FormRoutes : TStringDynArray; override;
  end;

implementation

uses dmServer, libtinyeditor;

{ --------------------------------------------------------------------
  TArticleForm
  --------------------------------------------------------------------}

procedure TArticleForm.SetupDatasets;

begin
  rdsArticle:=TSQLDBRestDataset.Create(Self);
  rdsArticle.Connection:=Server.RestConnection;
  rdsArticle.ResourceName:='articles';
  rdsArticle.AfterOpen:=@DoAfterArticleOpen;
  rdsComments:=TSQLDBRestDataset.Create(Self);
  rdsComments.Connection:=Server.RestConnection;
  rdsComments.ResourceName:='commentoverview';
  With rdsComments.Params.AddParam('c_article_fk') do
    begin
    DataType:=ftLargeInt;
    Enabled:=true;
    end;
  dsComments:=TDatasource.Create(Self);
  dsComments.Dataset:=rdsComments;
end;

procedure TArticleForm.SetupTemplates;

Const
   CommentTemplate =
    '<article class="media">' +
    '  <div class="media-content">' +
    '    <div class="content">' +
    '      <p>' +
    '        <strong>{{u_firstname}} {{u_lastname}}</strong>' +
    '          <small>{{c_created_on}}</small>' +
    '        <br>' +
    '          {{c_text}}' +
    '      </p>' +
    '    </div>' +
    '  </div>' +
    '</article>';


begin
  ltwComments:=TDBLoopTemplateWidget.Create(Self);
  ltwComments.Datasource:=dsComments;
  ltwComments.ItemTemplate:=CommentTemplate;
  ltwComments.ParentID:=divExistingComments.id;
end;

constructor TArticleForm.create(aOwner : TComponent);

begin
  inherited;
  BindElements;
  BindElementEvents;
  SetupDatasets;
  SetupTemplates;
end;



procedure TArticleForm.BindElements;

begin
  divNoArticles:=TJSHTMLElement(document.getelementByID('divNoArticles'));
  divTitle:=TJSHTMLElement(document.getelementByID('divTitle'));
  lblTitle:=TJSHTMLElement(document.getelementByID('lblTitle'));
  lblAuthor:=TJSHTMLElement(document.getelementByID('lblAuthor'));
  lblDate:=TJSHTMLElement(document.getelementByID('lblDate'));
  divEditTitle:=TJSHTMLElement(document.getelementByID('divEditTitle'));
  edtTitle:=TJSHTMLInputElement(document.getelementByID('edtTitle'));
  divArticleBlock:=TJSHTMLElement(document.getelementByID('divArticleBlock'));
  divArticle:=TJSHTMLElement(document.getelementByID('divArticle'));
  btnSave:=TJSHTMLButtonElement(document.getelementByID('btnSave'));
  divComments:=TJSHTMLElement(document.getelementByID('divComments'));
  divExistingComments:=TJSHTMLElement(document.getelementByID('divExistingComments'));

  editcomment:=TJSHTMLElement(document.getelementByID('editcomment'));
  btnAddComment:=TJSHTMLButtonElement(document.getelementByID('btnAddComment'));
end;

procedure TArticleForm.BindElementEvents;

begin
  btnSave.AddEventListener('click', @HandleSavePost);
  btnAddComment.AddEventListener('click', @HandleSaveComment);
end;

procedure TArticleForm.ShowRoute(const aURL: String; aRoute: TRoute;
  aParams: TStrings);

Var
  aID : String;

begin
  Inherited;
  aID:=aParams.Values['ArticleID'];
  if SameText(aID,'New') then
    begin
    FArticleID:=-1;
    rdsArticle.Params.AddParam('a_author_fk',true).asInteger:=-1;
    end
  else
    begin
    FArticleID:=StrToIntDef(aID,-1);
    rdsArticle.ResourceID:=IntToStr(FArticleID);
    end;
  rdsArticle.Load();
  if (FArticleID>0) then
    begin
    rdsComments.Params[0].asLargeInt:=FArticleID;
    rdsComments.Load();
    end
end;

class procedure TArticleForm.HideEl(aEl: TJSHTMLElement);

begin
  aEl.ClassList.add('is-hidden');
end;

procedure TArticleForm.HandleSavePost(Event: TJSEvent);
begin
  SaveData(rdsArticle);
  rdsArticle.Post;
  rdsArticle.ApplyUpdates;
end;

procedure TArticleForm.HandleSaveComment(Event: TJSEvent);
begin
  With rdsComments do
    begin
    Append;
    FieldByName('c_text').AsString:=editcomment.innerHTML;
    FieldByName('c_createdon').AsDatetime:=Now;
    FieldByName('c_author_fk').AsDateTime:=Server.UserInfo.ID;
    Post;
    ApplyUpdates;
    end;
end;

procedure TArticleForm.DoAfterArticleOpen(DataSet: TDataSet);

Var
  IsEdit : Boolean;
  S : String;

begin
  if Dataset.IsEmpty then
    begin
    if FarticleID>0 then
      begin
      HideEl(divArticle);
      HideEl(divTitle);
      HideEl(divEditTitle);
      HideEl(divComments);
      ShowEl(divNoArticles);
      end
    else
      begin
      lblAuthor.InnerText:=Server.UserInfo.Login;
      S:=FormatDateTime('yyyy-mm-dd hh:nn',Now);
      lblDate.InnerText:=S;
      edtTitle.value:='';
      divArticle.InnerHTML:='';
      SetupForEdit(True);
      Dataset.Append;
      end;
    end
  else
    begin
    isEdit:=Dataset.FieldByname('a_author_fk').AsInteger=Server.UserInfo.ID;
    SetupForEdit(isEdit);
    ShowData(Dataset);
    if IsEdit then
      Dataset.Edit;
    end;
  With Dataset.FieldByname('a_id') do
    ProviderFlags:=ProviderFlags+[pfInKey];
end;

procedure TArticleForm.SaveData(Dataset : TDataset);

begin
  With Dataset do
    begin
    FieldByName('a_created_on').AsDateTime:=Now;
    FieldByName('a_text').AsString:=divArticle.InnerHTML;
    FieldByName('a_title').AsString:=edtTitle.Value;
    FieldByName('a_author_fk').AsInteger:=Server.UserInfo.ID;
    end;
end;


procedure TArticleForm.ShowData(Dataset : TDataset);

var
  S : string;

begin
  With Dataset do
    begin
    S:=FieldByName('u_firstname').AsString + ' '
       + FieldByName('u_lastname').AsString;
    lblAuthor.InnerText:=S;
    S:=FormatDateTime('yyyy-mm-dd hh:nn',
                      FieldByName('a_created_on').AsDateTime);
    lblDate.InnerText:=S;
    S:=FieldByName('a_title').AsString;
    edtTitle.value:=S;
    lblTitle.InnerText:=S;
    S:=FieldByName('a_text').AsString;
    divArticle.InnerHTML:=S;
    end;
end;


class procedure TArticleForm.ShowEl(aEl: TJSHTMLElement);

begin
  aEl.ClassList.remove('is-hidden');
end;

procedure TArticleForm.SetupForEdit(IsEditing : Boolean);

begin
  if IsEditing then
    begin
    HideEl(divComments);
    HideEl(divTitle);
    ShowEl(divEditTitle);
    tinyEditor.transformToEditor(divArticle);
    ShowEl(btnSave);
    end
  else
    begin
    HideEl(divEditTitle);
    ShowEl(divComments);
    btnAddComment.disabled:=(Server.UserInfo.ID<=0);
    if not btnAddComment.disabled then
      begin
      editcomment.style.setProperty('min-height','10em;');
      tinyEditor.transformToEditor(editcomment);
      end;
    end;
end;

class function TArticleForm.FormRoutes: TStringDynArray;
begin
  Result:=['/article/:ArticleID'];
end;

end.
