unit frmarticlelist;
{$MODE ObjFPC}
{$H+}

interface

uses web, Types, SysUtils, Classes, frmbase, db, sqldbrestdataset, dbwebwidget;

type
  
  { --------------------------------------------------------------------
    TArticleListForm
    --------------------------------------------------------------------}
  
  { TArticleListForm }

  TArticleListForm = class(TBaseForm)
  private
    rdsAuthors : TSQLDBRestDataset;
    rdsArticles : TSQLDBRestDataset;
    dsArticles : TDatasource;
    ltwArticleList : TDBLoopTemplateWidget;
    procedure DoOpenArticles(DataSet: TDataSet);
    procedure DoOpenAuthors(DataSet: TDataSet);
    procedure SetupDatasets;
    procedure SetupTemplates;
  Published
    divSearch : TJSHTMLElement;
    selAuthor : TJSHTMLSelectElement;
    divNoArticles : TJSHTMLElement;
    divArticleList : TJSHTMLElement;
    divNewArticle : TJSHTMLElement;
    procedure DoSelectAuthor(Event : TJSEvent);
  Public
    constructor create(aOwner : TComponent); override;
    Class Function FormRoutes : TStringDynArray; override;
    Class Function FormHTMLFileName : String; override;

    procedure BindElements; virtual;
    procedure BindElementEvents; virtual;
  end;

implementation

uses dmserver;

{ --------------------------------------------------------------------
  TArticleListForm
  --------------------------------------------------------------------}

(*
procedure TArticleListForm.DoOpenAuthors(DataSet: TDataSet);

Const
  ItemTemplate = '<option value="%d">%s</option>'+sLineBreak;

var
  aHTML,aAuthor : String;
  FID,FFirstName,fLastname : TField;

begin
  aHTML:=Format(ItemTemplate,[-1,'All authors']);
  FID:=Dataset.FieldByName('u_id');
  FFirstName:=Dataset.FieldByName('u_firstname');
  FLastName:=Dataset.FieldByName('u_lastname');
  While not Dataset.EOF do
    begin
    aAuthor:=FFirstName.AsString+' '+fLastname.AsString;
    if (FID.AsInteger=Server.UserInfo.ID) then
      aAuthor:='You ('+aAuthor+')';
    aHTML:=aHTML+Format(ItemTemplate,[FID.asInteger,aAuthor]);
    Dataset.Next;
    end;
  selAuthor.InnerHTML:=aHTML;
end;
*)

procedure TArticleListForm.DoOpenAuthors(DataSet: TDataSet);

var
  aAuthor : String;
  FID,FFirstName,fLastname : TField;

  Function MakeOption(aID : NativeInt; const aName : String) : TJSHTMLOptionElement;

  begin
    Result:=TJSHTMLOptionElement(Document.createElement('option'));
    Result.innerText:=aName;
    Result.value:=IntToStr(aID);
  end;

begin
  selAuthor.InnerHTML:='';
  SelAuthor.append(MakeOption(-1,'All authors'));
  FID:=Dataset.FieldByName('u_id');
  FFirstName:=Dataset.FieldByName('u_firstname');
  FLastName:=Dataset.FieldByName('u_lastname');
  While not Dataset.EOF do
    begin
    aAuthor:=FFirstName.AsString+' '+fLastname.AsString;
    if (FID.AsInteger=Server.UserInfo.ID) then
      aAuthor:='You ('+aAuthor+')';
    selAuthor.append(MakeOption(FID.asInteger,aAuthor));
    Dataset.Next;
    end;
end;


procedure TArticleListForm.DoOpenArticles(DataSet: TDataSet);
begin
  if Dataset.IsEmpty then
    divNoArticles.classList.Remove('is-hidden')
  else
    divNoArticles.classList.Add('is-hidden');
end;

constructor TArticleListForm.create(aOwner : TComponent);

begin
  inherited;
  BindElements;
  BindElementEvents;
  SetupDatasets;
  SetupTemplates;
  if Server.UserInfo.ID>0 then
    divNewArticle.classList.remove('is-hidden');
end;

procedure TArticleListForm.SetupTemplates;

Const
   articleTemplate =
    '<article class="media">' +
    '  <div class="media-content">' +
    '    <div class="content">' +
    '      <p>' +
    '        <strong>{{u_firstname}} {{u_lastname}}</strong>' +
    '          <small>{{a_created_on}}</small>' +
    '        <br>' +
    '        <a href="#/article/{{a_id}}">' +
    '          {{a_title}}' +
    '        </a>' +
    '      </p>' +
    '    </div>' +
    '  </div>' +
    '</article>';


begin
  ltwArticleList:=TDBLoopTemplateWidget.Create(Self);
  ltwArticleList.Datasource:=dsArticles;
  ltwArticleList.ItemTemplate:=articleTemplate;
  ltwArticleList.ParentID:=divArticleList.ID;
end;

class function TArticleListForm.FormRoutes: TStringDynArray;

begin
  Result:=['/articles/'];
end;

class function TArticleListForm.FormHTMLFileName: String;
begin
  Result:='articles.html';
end;

procedure TArticleListForm.BindElements;

begin
  divSearch:=TJSHTMLElement(document.getelementByID('divSearch'));
  selAuthor:=TJSHTMLSelectElement(document.getelementByID('selAuthor'));
  divNoArticles:=TJSHTMLElement(document.getelementByID('divNoArticles'));
  divArticleList:=TJSHTMLElement(document.getelementByID('divArticleList'));
  divNewArticle:=TJSHTMLElement(document.getelementByID('divNewArticle'));
end;

procedure TArticleListForm.BindElementEvents;

begin
  selAuthor.AddEventListener('change', @DoSelectAuthor);
end;

procedure TArticleListForm.SetupDatasets;

begin
  rdsAuthors:=TSQLDBRestDataset.Create(Self);
  With rdsAuthors do
    begin
    Connection:=Server.RestConnection;
    ResourceName:='users';
    AfterOpen:=@DoOpenAuthors;
    Load;
    end;
  rdsArticles:=TSQLDBRestDataset.Create(Self);
  With rdsArticles do
    begin
    Connection:=Server.RestConnection;
    ResourceName:='articleoverview';
    AfterOpen:=@DoOpenArticles;
    Params.AddParam('sort',true).AsString:='a_created_on desc';
    Params.AddParam('a_author_fk',False);
    end;
  dsArticles:=TDatasource.Create(Self);
  dsArticles.Dataset:=rdsArticles;
  rdsArticles.Load;
end;

procedure TArticleListForm.DoSelectAuthor(Event: TJSEvent);

Var
  aAuthorID : Integer;

begin
  aAuthorID:=StrToIntDef(selAuthor.value,-1);
  With rdsArticles.ParamByName('a_author_fk') do
    begin
    AsInteger:=aAuthorID;
    Enabled:=(aAuthorID<>-1);
    end;
  rdsArticles.Close;
  rdsArticles.Load;
end;

end.
