create sequence seq_users;

create table users (
  u_id bigint not null default nextval('seq_users'),
  u_firstname varchar(50) NOT NULL,
  u_lastname varchar(50) NOT NULL,
  u_login varchar(127) not null,
  u_password varchar(127) not null,
  constraint pk_users primary key (u_id)
);

create unique index udx_users on users (lower(u_login))

create sequence seq_articles;

create table articles (
  a_id bigint not null default nextval('seq_articles'),
  a_created_on timestamp not null default CURRENT_TIMESTAMP,
  a_author_fk bigint not null,
  a_title varchar(255) not null,
  a_text text not null,
  constraint pk_articles primary key (a_id)
);

create index idx_articles_created on articles (a_created_on);

create sequence seq_comments;

create table comments (
  c_id bigint not null default nextval('seq_comments'),
  c_created_on timestamp not null default CURRENT_TIMESTAMP,
  c_article_fk bigint not null,
  c_author_fk bigint not null,
  c_text text not null,
  constraint pk_comments primary key (c_id)
)

create index idx_comments_created on comments (c_created_on);

alter table articles add constraint fk_articles_users foreign key (a_author_fk) references users (u_id) on delete cascade;
alter table comments add constraint fk_comments_users foreign key (c_author_fk) references users (u_id) on delete cascade;
alter table comments add constraint fk_comments_articles foreign key (c_article_fk) references articles(a_id) on delete cascade;


GRANT INSERT, REFERENCES, SELECT, UPDATE, DELETE, TRIGGER, TRUNCATE ON TABLE public.users TO blogger;
GRANT INSERT, REFERENCES, SELECT, UPDATE, DELETE, TRIGGER, TRUNCATE ON TABLE public.articles TO blogger;
GRANT INSERT, REFERENCES, SELECT, UPDATE, DELETE, TRIGGER, TRUNCATE ON TABLE public.comments TO blogger;

